import('lang.Object')
import('model.Component')
import('model.toolbar.header.ToolbarHeader')
import('model.toolbar.section.Section')
import('model.toolbar.AddSection')

---@class Toolbar : Component
Toolbar = extends(Component)
Toolbar.__class = "Toolbar"

function Toolbar:create(parent)
    local root = Toolbar:createRootElement(parent, {
        type = "frame",
        direction = "vertical",
        style = "toolbar" })
    ToolbarHeader:create(root)
    local content = root.add {
        type = "flow",
        name = "content",
        direction = "vertical",
        style = "toolbar_content" }
    local sections = content.add {
        type = "flow",
        name = "sections",
        direction = "vertical",
        style = "toolbar_content_sections" }
    Section:create(sections)
    AddSection:create(content)
    return Toolbar:new(root)
end

function Toolbar:new(element)
    return Toolbar:super(Component:new(element))
end

function Toolbar:childrenClasses()
    return { ToolbarHeader, Section, AddSection }
end

---@public
function Toolbar:centerOnScreen()
    local resolution = game.players[self:root().player_index].display_resolution
    self:root().location = { resolution.width / 2 - 113, resolution.height / 2 }
end

---@public
function Toolbar:addSection()
    Section:create(self:sectionsElement())
end

---@public
function Toolbar:collapse()
    self:contentElement().visible = false
end

---@public
function Toolbar:expand()
    self:contentElement().visible = true
end

function Toolbar:sectionsElement()
    return self:contentElement().sections
end

function Toolbar:contentElement()
    return self:root().content
end

---@public
---@return Section[]
function Toolbar:sections()
    return Section:findDown(self:root())
end
