import('model.Component')
import('model.toolbar.header.Lock')
import('model.toolbar.header.Unlock')
import('model.toolbar.header.CollapseToolbar')
import('model.toolbar.header.ExpandToolbar')
import('model.toolbar.header.DeleteToolbar')
import('model.toolbar.header.CancelDeleteToolbar')
import('model.toolbar.header.ConfirmDeleteToolbar')

---@class ToolbarHeader : Component
ToolbarHeader = extends(Component)
ToolbarHeader.__class = "ToolbarHeader"

function ToolbarHeader:create(parent)
    local root = ToolbarHeader:createRootElement(parent, {
        type = "flow",
        direction = "horizontal",
        style = "toolbar_header" })
    local toolbar = Toolbar:findUp(parent):root()
    root.drag_target = toolbar
    Lock:create(root)
    root.add {
        type = "empty-widget",
        style = "toolbar_drag" }
        .drag_target = toolbar
    CollapseToolbar:create(root)
    DeleteToolbar:create(root)
    return ToolbarHeader:new(root)
end

function ToolbarHeader:new(element)
    return ToolbarHeader:super(Component:new(element))
end

function ToolbarHeader:childrenClasses()
    return { Lock, Unlock,
             CollapseToolbar, ExpandToolbar,
             DeleteToolbar, CancelDeleteToolbar, ConfirmDeleteToolbar }
end

function ToolbarHeader:askForDeletion()
    DeleteToolbar:findDownFirst(self:root()):delete()
    ConfirmDeleteToolbar:create(self:root())
    CancelDeleteToolbar:create(self:root())
end

function ToolbarHeader:cancelDeletion()
    CancelDeleteToolbar:findDownFirst(self:root()):delete()
    ConfirmDeleteToolbar:findDownFirst(self:root()):delete()
    DeleteToolbar:create(self:root())
end
