import('model.Component')
import('model.toolbar.section.table.header.TableHeader')
import('model.toolbar.section.table.Row')
import('model.toolbar.section.table.slots.Empty')

---@class Table : Component
Table = extends(Component)
Table.__class = "Table"

function Table:create(parent)
    local toolbar = Toolbar:findUp(parent):root()
    local root = Table:createRootElement(parent, {
        type = "flow",
        direction = "vertical",
        style = "toolbar_content_sections_section_content_table" })
    root.drag_target = toolbar

    TableHeader:create(root)
    root.drag_target = toolbar

    local content = root.add {
        type = "frame",
        name = "content",
        direction = "vertical",
        style = "toolbar_content_sections_section_content_table_content" }
    content.drag_target = toolbar

    local table = Table:new(root)
    table:addRow()
    return table
end

function Table:new(element)
    return Table:super(Component:new(element))
end

function Table:childrenClasses()
    return { TableHeader, Row }
end

---@public
function Table:addColumn()
    for _, row in ipairs(self:rows()) do
        row:addColumn()
    end
end

---@public
function Table:tryRemovingLastColumn()
    if self:columnsCount() > 1 and not self:hasOccupiedSlotsInLastColumn() then
        for _, row in ipairs(self:rows()) do
            row:deleteLastSlot()
        end
    end
end

---@private
---@return boolean
function Table:hasOccupiedSlotsInLastColumn()
    for _, row in ipairs(self:rows()) do
        if row:hasOccupiedLastSlot() then
            return true
        end
    end
    return false
end

---@private
---@return number
function Table:columnsCount()
    return self:firstRow():slotsCount()
end

---@public
function Table:addRow()
    Row:create(self:contentElement())
end

---@public
function Table:tryRemovingLastRow()
    if self:rowsCount() > 1 then
        local lastRow = self:lastRow()
        if not lastRow:hasOccupiedSlots() then
            lastRow:delete()
        end
    end
end

---@private
---@return Row
function Table:firstRow()
    return self:rows()[1]
end

---@private
---@return Row
function Table:lastRow()
    return self:rows()[self:rowsCount()]
end

---@private
---@return number
function Table:rowsCount()
    return #self:rows()
end

---@private
---@return Row[]
function Table:rows()
    local rows = {}
    for _, row in ipairs(self:contentElement().children) do
        table.insert(rows, Row:new(row))
    end
    return rows
end

---@return TableHeader
function Table:header()
    return TableHeader:findDownFirst(self:root())
end

---@return LuaGuiElement
function Table:contentElement()
    return self:root().content
end
